/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvMath.h"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_yQJof3FJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rBP3hb1y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RG3u4O1e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wMoPberZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QOWsGukx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1wy3Vqym_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wFwq01xS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_FJBoSgFj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PadjUXn4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jzJS86K5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vH4DPbcQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_agmGdDxO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_zVXmqqMH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fKskBE3h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XpxGRBf6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CMmAP5Qo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_408XcpMD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PnhcHTDM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IkC5UmIv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4OhF8Ikl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_eCPLqmSb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VS0p7EHx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_WVM98VYA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_QwhW7Ix5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_m1whdqts_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_jylTwfPx_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_b1KXgUu6;
  SignalLine sLine_OwYfae0r;
  SignalLine sLine_rBM6205H;
  ControlBinop cBinop_yQJof3FJ;
  ControlBinop cBinop_rBP3hb1y;
  ControlBinop cBinop_RG3u4O1e;
  ControlBinop cBinop_wMoPberZ;
  ControlBinop cBinop_wFwq01xS;
  ControlBinop cBinop_PadjUXn4;
  ControlBinop cBinop_jzJS86K5;
  ControlBinop cBinop_vH4DPbcQ;
  ControlVar cVar_agmGdDxO;
  ControlVar cVar_zVXmqqMH;
  SignalVarf sVarf_jOXBwU9t;
  SignalVarf sVarf_YJ12DRb8;
  SignalVarf sVarf_EQrn3iCq;
  SignalVarf sVarf_qU9jcaGq;
  ControlBinop cBinop_PnhcHTDM;
  ControlBinop cBinop_VS0p7EHx;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
